//------------------------------------------------------------------------------
//
//  npfile.h
//
//  ANTz application based on NPE is free software
//	NPE is the Neural Physics Engine that drives the interactions
//
//	ANTz hosted at http://temporalzone.com and NPE at http://neuralphysics.org
//
//	Copyright (c) 2010-2011, Shane Saxon
//	All rights reserved.
//	Redistribution and use in source and binary forms, with or without 
//	modification, are permitted provided that the following conditions are met:
//		Redistributions of source code must retain the above copyright notice, 
//		this list of conditions and the following disclaimer.
//		Redistributions in binary form must reproduce the above copyright 
//		notice, this list of conditions and the following disclaimer in the 
//		documentation and/or other materials provided with the distribution.
//		Neither the name of the Shane Saxon nor the names of its contributors 
//		may be used to endorse or promote products derived from this software 
//		without specific prior written permission.
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
//	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
//	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
//	POSSIBILITY OF SUCH DAMAGE.
//
//------------------------------------------------------------------------------

#ifndef NPFILE_H_
#define NPFILE_H_


#include <stdio.h>
#include "../npdata.h"
#include "../data/npmapfile.h"

#define kNPfileMax 4096

//------------------------------------------------------------------------------
void npInitFile (void* dataRef);	// init the file io system
void npCloseFile (void* dataRef);	// for init and shutdown, not closing a file

// these functions wrap the standard C file methods on Linux, MSW and OSX
// opens or creates the file in the specified mode "rb+" read/write binary..
FILE* npFileOpen (const char* fileName, const char* mode, void* dataRef);
FILE* npFileNew (const char* fileName, const char* mode, void* dataRef);
FILE* npFileDialog (const char* fileName, int dialogType, void* dataRef);

// if not saved then opens SaveAs dialog before reseting app
int npFileClose (FILE* file, void* dataRef);

// wrappers for fread(), fwrite() and rewind()
//void npFileRead (void* writeBuffer, int wordSize, int size, void* dataRef);
int npFileRead (void* readBuffer, int elementSize, int elementCount, 
				 FILE* file, void* dataRef);
int npFileWrite (const void* str, int wordSize, int size, FILE* file, void* dataRef);

void npFileRewind (FILE* file);
int npFileSeek (FILE* file, long offSet, int origin);
long npFileTell (FILE* file);

// 
void npSave (FILE* file, void* dataRef);
void npSaveAs (int size, void* dataRef);

void npImport (const char* fileName, void* dataRef);
void npExport (int size, void* dataRef);

// file and directory management

void npGetOpenFilePath (char* buffer, int* size, void* dataRef);
void npGetCWD (char* buffer, int* size);			//get Current Working Directory

void npListDir (int size, void* dataRef);

void npMakeDir (int size, void* dataRef);
void npDelDir (int size, void* dataRef);

void npGetFileAttrib (int size, void* dataRef);
void npSetFileAttrib (int size, void* dataRef);

void npTestFile (void* fileName, int wordSize, int size, void* dataRef);

void npOpenURL (const char* url, void* dataRef);
void npOpenApp (const char* fileName, void* dataRef);
void npOpenAntz (const char* fileName, void* dataRef);


#endif

