//------------------------------------------------------------------------------
//
//	npmouse.c
//
//  ANTz application based on NPE is free software
//	NPE is the Neural Physics Engine that drives the interactions
//
//	ANTz hosted at http://temporalzone.com and NPE at http://neuralphysics.org
//
//	Copyright (c) 2010-2011, Shane Saxon
//	All rights reserved.
//	Redistribution and use in source and binary forms, with or without 
//	modification, are permitted provided that the following conditions are met:
//		Redistributions of source code must retain the above copyright notice, 
//		this list of conditions and the following disclaimer.
//		Redistributions in binary form must reproduce the above copyright 
//		notice, this list of conditions and the following disclaimer in the 
//		documentation and/or other materials provided with the distribution.
//		Neither the name of the Shane Saxon nor the names of its contributors 
//		may be used to endorse or promote products derived from this software 
//		without specific prior written permission.
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
//	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
//	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
//	POSSIBILITY OF SUCH DAMAGE.
//
//------------------------------------------------------------------------------

#include "npmouse.h"

#include "../npctrl.h"
#include "../npdata.h"

#include "npgl.h"



//------------------------------------------------------------------------------
void npInitMouse (void* dataRef)
{
	pData data = (pData) dataRef;

	data->io.mouse.x = 0;						//raw coordinate
	data->io.mouse.y = 0;
	data->io.mouse.z = 0;						//typically the scroll wheel

	data->io.mouse.buttonDown = NOBUTTON;	

	data->io.mouse.buttonL = 0;					//true when pressed
	data->io.mouse.buttonC = 0;
	data->io.mouse.buttonR = 0;

	data->io.mouse.screenX = 0;					//screen position in pixels
	data->io.mouse.screenY = 0;

	data->io.mouse.previousX = 0;
	data->io.mouse.previousY = 0;
	data->io.mouse.previousZ = 0;

	data->io.mouse.theta = 0.0f;
	data->io.mouse.zeta = 0.0f;

	data->io.mouse.cmDX = 0.0f;
	data->io.mouse.cmDY = 0.0f;
	
	data->io.mouse.tX = 0.0f;
	data->io.mouse.tY = 0.0f;
	data->io.mouse.tZ = 0.0f;

	data->io.mouse.deltaX = 0;					//mouse vector
	data->io.mouse.deltaY = 0;					//clears every mouse event
	data->io.mouse.deltaZ = 0;

	data->io.mouse.accumulateX = 0;			//clears each NPE period
	data->io.mouse.accumulateY = 0;
	data->io.mouse.accumulateZ = 0;

	data->io.mouse.size = sizeof(NPmouse);

	return;
}


//------------------------------------------------------------------------------
void npMouseEvent (int button, int state, int x, int y)
{
	pData data = (pData) npGetDataRef();
	
//	npMouseEventGL (button, state, x, y);

	data->io.mouse.previousX = x;
	data->io.mouse.previousY = y;	


	if (button == GLUT_LEFT_BUTTON && state == GLUT_UP)
	{
		data->io.mouseDeltaX = 0.0f;
		data->io.mouseDeltaY = 0.0f;
	}

	if( state == GLUT_DOWN )	//Mouse Click Down (Press)
	{
		data->io.mouse.buttonDown = button;

		PickPass( true );
		npGLDrawScene( data );		//optimize to draw only picked region, debug zz
		y = height - y;
		npPick( x, y );
		npGLDrawScene( data );							//zz needed to select object... debug
		PickPass( false );
	}

	if( state == GLUT_UP )						//Mouse Click Up (Release)
	{	
		data->io.picking = false;
		PickPass( false );
		data->io.mouse.buttonDown = NOBUTTON;
		data->io.mouse.cmDY = 0.0f;
	}
}


//------------------------------------------------------------------------------
void npMouseMotion (int x, int y)
{
	float dX = 0;
	float dY = 0;
	
	pData data = (pData) npGetDataRef();

	dX = data->io.mouseDeltaX = (GLfloat)(x - data->io.mouse.previousX);
	dY = data->io.mouseDeltaY = (GLfloat)(y - data->io.mouse.previousY);


	if( !data->io.picking )
	{
		if (data->map.nodeRootCount > kNPpinRoot)
		{
			npCtrlCommand (kGraphSelectAllToggle, data);	//zz debug not efficient
			npCtrlCommand (kGraphCamera, data);
		}

		if( data->io.mouse.buttonDown == GLUT_LEFT_BUTTON )
		{
			data->io.mouse.theta += (float)dX / 450.0f;
			data->io.mouse.zeta += (float)dY / 600.0f;
		}

		if( data->io.mouse.buttonDown == GLUT_RIGHT_BUTTON )
		{
			data->io.mouse.cmDX += (float)dX / 200.0f;
			data->io.mouse.cmDY += (float)dY / 200.0f;

			data->io.mouse.theta += (float)dX / 450.0f;
		}
	}

	data->io.mouse.previousX = x;
	data->io.mouse.previousY = y;
}

